use std::sync::Arc;

use axum::{extract::State, http::StatusCode, response::IntoResponse, routing::get, Router};
use data::Data;
use maud::{html, Markup};

pub mod data;

pub struct AppState {
    pub data: Data,
}

pub struct AppError(anyhow::Error);

impl IntoResponse for AppError {
    fn into_response(self) -> axum::response::Response {
        (
            StatusCode::INTERNAL_SERVER_ERROR,
            format!("Something went wrong: {}", self.0),
        )
            .into_response()
    }
}

impl<E> From<E> for AppError
where
    E: Into<anyhow::Error>,
{
    fn from(err: E) -> Self {
        Self(err.into())
    }
}

#[axum::debug_handler]
async fn main_handler(state: State<Arc<AppState>>) -> Result<Markup, AppError> {
    println!("{:?}", state.data.db_data.fish);
    let values = state.data.db_data.fish.values();
    Ok(html! {
        h1 { "Hello!" }
        @for fish in values {
            li { (fish.id) }
        }
    })
}

#[tokio::main]
async fn main() {
    let app = Router::new()
        .route("/", get(main_handler))
        .with_state(Arc::new(AppState { data: Data::new() }));
    let listener = tokio::net::TcpListener::bind("0.0.0.0:3000").await.unwrap();
    println!("Listening on http://localhost:3000!");
    axum::serve(listener, app).await.unwrap();
}
